package com.fossgalaxy.games.tbs.editor.EditorView;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.editor.PlacementType;
import com.fossgalaxy.games.tbs.editor.tools.Tool;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.io.map2.EntityDef;
import com.fossgalaxy.games.tbs.io.map2.MapDef;
import com.fossgalaxy.games.tbs.parameters.EntityType;
import com.fossgalaxy.games.tbs.parameters.GameSettings;
import com.fossgalaxy.games.tbs.parameters.ResourceType;
import com.fossgalaxy.games.tbs.parameters.TerrainType;
import com.fossgalaxy.games.tbs.ui.UIModel;
import org.codetome.hexameter.core.api.CubeCoordinate;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class EditorModel {

    // map related
    private GameSettings settings;
    private MapDef def;

    //editor related
    private Tool tool;
    private PlacementType placementType;
    private boolean showingLocations;

    //selected types
    private EntityType selectedEntity;
    private ResourceType selectedResource;
    private TerrainType selectedTerrain;
    private int owner;

    private EntityDef selectedEntityDef;

    private List<EditorListener> listeners;
    private CubeCoordinate selected;
    private Entity currHighlight;

    public EditorModel(GameSettings settings){
        this.settings = settings;
        this.listeners = new ArrayList<>();
    }

    public void addEditorListener(EditorListener listener){
        this.listeners.add(listener);
    }

    protected void fireEntityChanged(EntityDef def){
        for (EditorListener listener : listeners){
            listener.onEntitySelected(def);
        }
    }

    protected void fireLocationSelected(CubeCoordinate loc){
        for (EditorListener listener : listeners){
            listener.onLocationSelected(loc);
        }
    }

    protected void fireMapDefLoaded(){
        for (EditorListener listener : listeners){
            listener.onMapDefLoaded(def);
        }
    }

    public void setEntityDef(EntityDef def){
        this.selectedEntityDef = def;
        fireEntityChanged(def);
    }

    public EntityDef getEntityDef(){
        return selectedEntityDef;
    }

    public void setState(GameState s){
        throw new IllegalArgumentException("Setting state in editor is not supported");
    }

    public void setSettings(GameSettings settings){
        this.settings = settings;
    }

    public GameSettings getSettings() {
        return settings;
    }

    public void setTool(Tool tool){
        this.tool = tool;
    }

    public void setMapDef(MapDef def){
        this.def = def;
        fireMapDefLoaded();
    }

    public MapDef getDef() {
        return def;
    }

    public Tool getTool() {
        return tool;
    }

    public void setSelectedEntity(EntityType selectedEntity) {
        this.placementType = PlacementType.ENTITY;
        this.selectedEntity = selectedEntity;
    }

    public void setSelectedResource(ResourceType selectedResource) {
        this.placementType = PlacementType.RESOURCE;
        this.selectedResource = selectedResource;
    }

    public void setSelectedTerrain(TerrainType selectedTerrain) {
        this.placementType = PlacementType.TERRAIN;
        this.selectedTerrain = selectedTerrain;
    }

    public PlacementType getType() {
        return placementType;
    }

    public TerrainType getSelectedTerrain() {
        return selectedTerrain;
    }

    public ResourceType getSelectedResource() {
        return selectedResource;
    }

    public EntityType getSelectedEntity() {
        return selectedEntity;
    }

    public void setOwner(int owner) {
        this.owner = owner;
    }

    public int getOwner() {
        return owner;
    }

    public void setSelected(CubeCoordinate selected) {
        this.selected = selected;
        fireLocationSelected(selected);
    }

    public CubeCoordinate pix2cube(Point point) {
        GameState s = def.buildState(settings);
        return s.pix2cube(point);
    }

    public CubeCoordinate getSelected() {
        return selected;
    }

    public void setShowingLocations(boolean s) {
        this.showingLocations = s;
    }

    public boolean isShowingLocations() {
        return showingLocations;
    }
}
